<?php

namespace App\Models;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Validator;

class Chat extends Model {   

    use SoftDeletes;    
    
    protected $fillable = [
        'msg', 'user_id',
    ];
    
    protected $hidden = [        
        'user_id',
    ];           
    
    protected static $rules = [
        'id'      => 'exists:news|sometimes|required',
        'msg'     => 'string|between:1,256|sometimes|required',        
        'user_id' => 'exists:users,id|sometimes|required',
    ];

    public function user()
    {
        return $this->belongsTo('App\Models\User');            
    }
    
    /**
     * Is $parameters valid ? 
     * ($parameters are tested against $rules)
     * 
     * @param array $parameters
     * @return boolean
     */
    public static function isValid($parameters)
    {        
        $validator = Validator::make($parameters, self::$rules);        
        return $validator->passes();        
    }
       
}
