<?php

namespace App\Models;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\SoftDeletes;
use Validator;

class News extends Model {   

    use SoftDeletes;    
    
    protected $hidden = [        
        'user_id',
    ];   
    
    protected $fillable = [
        'title', 'body',
    ];
    
    protected static $rules = [
        'id'      => 'exists:news|sometimes|required',
        'title'   => 'sometimes|string|between:1,120',
        'body'    => 'sometimes|required|string|between:1,10000',
        'user_id' => 'exists:users,id|sometimes|required',
    ];

    public function user()
    {
        return $this->belongsTo('App\Models\User');            
    }
    
    /**
     * Is $parameters valid ? 
     * ($parameters are tested against $rules)
     * 
     * @param array $parameters
     * @return boolean
     */
    public static function isValid($parameters)
    {        
        $validator = Validator::make($parameters, self::$rules);        
        return $validator->passes();
        
    }
       
}
