<?php
namespace App\Services;
use Collator;

class Normalize {
    const PRIMARY = Collator::PRIMARY;
    const SECONDARY = Collator::SECONDARY;
    const TERTIARY = Collator::TERTIARY;

    public function nfc($str)
    {
        // validation/convertion utf8
        return normalizer_normalize($str, \Normalizer::FORM_C);
    }
    
    public function nfd($str)
    {
        return normalizer_normalize($str, \Normalizer::FORM_D);
    }       
    
    public function compare($str1, $str2, $strength = Collator::TERTIARY, $local = "en_EN")
    {        
        $collator = new Collator($local);
        $collator->setStrength($strength);
        return $collator->compare($str1, $str2);
    }
    
    public function trimBerforeFirstUppercaseLetter($str)
    {
        $str = $this->nfd($str);
        return preg_replace('/^([\P{Lu}])*(\p{Lu})/u', '${2}', $str);         
    }

    public function removeDicaritic($str)
    {
        $str = $this->nfd($str);
        return preg_replace('/\p{M}/u', '', $str);
    }
    
    public function getOnlyDigitsOrLetters($str)
    {        
        return preg_replace('/[^\p{L}|\d]/u', '', $this->nfd($str));        
    }
    
    public function purifyStr($str)
    {   
        $str = $this->removeDicaritic($str);        
        $str = $this->getOnlyDigitsOrLetters($str);
        $str = iconv("UTF-8", "ASCII//TRANSLIT//IGNORE", $str);
        return $str;
    }
    
    public function removeLigature($str)
    {
        return transliterator_transliterate('Any-Latin; Latin-ASCII;', $str);
    }
    
    public function isPalindrom($str)
    {
        $str = $this->nfd($str);  
        $str = $this->removeLigature($str);
        $str = $this->getOnlyDigitsOrLetters($str);
        $start = 0;
        $end = grapheme_strlen($str) - 1;
        while (
            $start < $end && 
            $this->compare(
                 grapheme_substr($str, $start, 1), 
                 grapheme_substr($str, $end, 1),
                 self::PRIMARY
             ) == 0
        ){
            $start++;
            $end--;
        }
        return $start >= $end;
    }
    
}