<?php
namespace App\Ws;
use Ratchet\MessageComponentInterface;
use Ratchet\ConnectionInterface;
use Request;

class Router implements MessageComponentInterface {
    protected $clients;

    public function __construct() {
        $this->clients = new \SplObjectStorage();
    }

    public function onOpen(ConnectionInterface $conn) {              
        $this->clients->attach($conn);        
        echo "New connection! ({$conn->resourceId})\n";
    }

    public function onMessage(ConnectionInterface $from, $msg) {
        $data = json_decode($msg, true);
        if (json_last_error() != JSON_ERROR_NONE) {
            $this->sendData('error', 'JSON invalid', $from);
            return;
        }
        // Le message doit contenir une action
        if (!isset($data['action'])) {
            $this->sendData('error', 'action is missing', $from);
            return;
        }
        // Le message doit contenir un controller
        if (!isset($data['controller'])) {            
            $this->sendData('error', 'controller is missing', $from);
            return;
        }
        // la classe du controller doit exister
        $controllerClassName = __NAMESPACE__ . "\\Controllers\\" . ucfirst($data['controller']);
        if (!class_exists($controllerClassName)) {            
            $this->sendData('error', 'controller invalid', $from);
            return;
        }
        $controller = new $controllerClassName();
        // L'action doit être existante sur le controller
        if (!method_exists($controller, $data['action'])) {
            $this->sendData('error', 'action invalid', $from);            
            return;
        }
        // Utilise Request pour le passage des para. au controller
        Request::replace($data);
        // Execute l'action en fournissant le client actuel et tous les clients
        call_user_func_array(
            [$controller, $data['action']],
            [$this, $from]
        );
    }
    
    public function send($msg, $clients = []){
        if (!is_array($clients) && !($clients instanceof \SplObjectStorage)) {                                                    
            $clients = [$clients];
        }
        foreach ($clients as $client) {
            if (!($client instanceof ConnectionInterface)) {
                throw new \Exception("Wrong parameter type: clients");
            }
            $client->send($msg);            
        }
    }
    
    public function sendJson($msg, $clients = []){
        $this->send(json_encode($msg), $clients);
    }
    
    public function sendData($action, $data, $clients){
        $this->sendJson([
            'action' => $action,
            'data'  => $data,
        ], $clients);
    }
    
    public function sendDataToAll($action, $data){        
        $this->sendComem($action, $data, $this->clients);
    }

    public function onClose(ConnectionInterface $conn) {        
        $this->clients->detach($conn);
        echo "Connection {$conn->resourceId} has disconnected\n";
    }

    public function onError(ConnectionInterface $conn, \Exception $e) {
        echo "An error has occurred: {$e->getMessage()}\n";
        $conn->close();
    }
    
    public function getClients()
    {
        return $this->clients;
    }
        
}
