<?php
namespace App\Ws;
use Auth;
use Request;

class RouterAuth extends Router {
    protected $authClients;
    
    public function __construct() {
        $this->authClients = new \SplObjectStorage();
        parent::__construct();
    }
   
    public function onMessage(\Ratchet\ConnectionInterface $from, $msg) {
        $data = json_decode($msg, true);
        if (json_last_error() != JSON_ERROR_NONE) {
            $this->sendData('error', 'JSON invalid', $from);
            return;
        }
        // Le message doit contenir une action
        if (!isset($data['action'])) {
            $this->sendData('error', 'action is missing', $from);
            return;
        }
        // Le message doit contenir un controller
        if (!isset($data['controller'])) {            
            $this->sendData('error', 'controller is missing', $from);
            return;
        }
        Request::replace($data);
                
        // la classe du controller doit exister
        $controllerClassName = __NAMESPACE__ . "\\Controllers\\" . ucfirst($data['controller']);
        if (!class_exists($controllerClassName)) {            
            $this->sendData('error', 'controller invalid', $from);
            return;
        }
        $controller = new $controllerClassName();
        // L'action doit être existante sur le controller
        if (!method_exists($controller, $data['action'])) {
            $this->sendData('error', 'action invalid', $from);            
            return;
        }
        // Auth déjà faites ?
        // Gestion du cas particulier de l'auth
        if ($data['controller'] != "auth" && $data['action'] != "login" && !$this->authClients->contains($from)) {           
            $this->sendData('error', 'you must be log for this action', $from);            
            return;
        }
        // Dispatcher
        call_user_func_array(
            [$controller, $data['action']],
            [$this, $from]
        );                                                
    }      
    
    public function addAuthClient($client)
    {
        $this->authClients->attach($client);
    }
    
    public function removeAuthClient($client)
    {
        $this->authClients->detach($client);
    }
    
    public function sendDataToAllAuth($action, $data)
    {        
        $this->sendData($action, $data, $this->authClients);
    }
    
    public function getAuthClients() {
        return $this->authClients;
    }
        
}
